==> node-validator-tool-1.0.10.tgz: package/src/index.ts <==
'use strict';

// node modules
import * as util from 'util';

interface MetaInfo
{
    required: boolean;
    type: string;
}

export type Dictionary<T> = { [name: string]: T };
export type ValidationFunction = (value: string) => boolean;
export type RuleValidator = RegExp | ValidationFunction | String;

/**
 * Module factory
 */
export function create(rules: Dictionary<RuleValidator>): Validator
{
    return new Validator(rules);
}

export class Validator
{
    public namedRules: Dictionary<RuleValidator> = {
        'uuid': /^([a-z0-9]{8}-)([a-z0-9]{4}-){3}([a-z0-9]{12})$/,
        'email': /\S+@\S+/,
        'datetime': /^\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}:\d{2}(\.\d+)?$/,
        'text': /^[\S\s]+$/,
        'number': /^\d+$/,
        'bool': /^t(rue)?|f(alse)?$/i,
        'abbr': /^[A-Z]+$/,
        'alphanum': /^[a-z0-9]+$/i,
        'gender': /^[mf]$/i,
        'zip': /^[0-9\s]+$/i,
        'phone': /^\+?[0-9\s]+$/,
        'countrycode': /^[A-Z]{2}$/,
    };
    public rules: Dictionary<RuleValidator> = {};
    public error: Dictionary<string> = {};

    /**
     * Module constructor
     */
    constructor(rules?: Dictionary<RuleValidator>)
    {
        // add custom rules
        if (util.isObject(rules)) {
            for (let name in rules) {
                this.addRule(name, rules[name]);
            }
        }
    }

    /**
     * Validation method
     */
    public validate(value: any, validator: string | RuleValidator): boolean
    {
        // no validation needed
        if (this._isEmpty(validator)) {
            return true;
        }

        let item = this._meta(validator);

        // skip empty value
        if (this._isEmpty(value)) {
            return !item.required;
        }

        // validation
        switch (item.type) {
            case 'regexp':
                return (validator as RegExp).test(value);

            case 'lambda':
                return (validator as ValidationFunction)(value);

            default:
                let rule = this.rules[item.type] || this.namedRules[item.type];

                if (rule === undefined) {
                    throw new Error(util.format('Unsupported validation type: %s', item.type));
                }

                if (util.isRegExp(rule)) {                    
                    return (rule as RegExp).test(value);
                }

                if (util.isFunction(rule)) {
                    return (rule as ValidationFunction)(value);
                }
        }
    }

    /**
     * Run all rules against respective item properties
     */
    public validateAll(item: any): boolean
    {
        this.error = {};
        
        let name: string,
            validator: RuleValidator,
            required: boolean,
            value: any,
            pass: boolean = true;

        for (name in this.rules) {
            validator = this.rules[name];
            required = this._meta(validator).required;
            value = this._resolve(item, name, required);

            if (this.validate(value, validator) === false) {
                this.error[name] = (value === undefined)
                    ? `Missing property: ${name}`
                    : `Validation error: given "${value}", expected "${validator}"`;
                pass = false;
            }
        }

        return pass;
    }

    public hasErrors(): boolean
    {
        return Object.keys(this.error).length > 0;
    }

    public getErrors(): Dictionary<string>
    {
        return this.error;
    }

    /**
     * Adds new rule
     */
    public addRule(name: string, validator: RuleValidator): Validator
    {
        if (!util.isString(name)) {
            throw new Error(util.format('Parameter "name" must be string, %s given', typeof name));
        }

        if (util.isNull(validator)) {
            return this;    // skip
        }

        if (!util.isRegExp(validator) && !util.isFunction(validator) && !util.isString(validator)) {
            throw new Error(util.format('Parameter "validator" must be either RegExp, Function or String (named validator), %s given', typeof validator));
        }

        const meta = this._meta(validator);

        if (util.isString(validator) && this.namedRules[meta.type] === undefined) {
            throw new Error(`Parameter "validator": invalid named validator "${validator}"`);
        }

        this.rules[name] = util.isString(validator) 
            ? this._getNamedRule(meta)
            : validator;

        return this;
    }

    /**
     * Removes a rule
     */
    public removeRule(name: string): Validator
    {
        delete this.rules[name];
        return this;
    }

    /**
     * Returns whether this validator contains a rule of the specified name
     */
    public hasRule(name: string): boolean
    {
        return !!this.rules[name];
    }


    // Helper functions

    /**
     * Returns named rule
     */
    private _getNamedRule(item: MetaInfo): RuleValidator
    {
        return item.type + (item.required ? '' : ':null');
    }

    /**
     * Parse meta info from supplied type
     */
    private _meta(validator: string | RuleValidator): MetaInfo
    {
        let item: MetaInfo = {
            type: null,
            required: null,
        };

        // validation not needed
        if (this._isEmpty(validator)) {
            return item;
        }

        // named validator
        if (util.isString(validator)) {
            let part = (validator as string).split(':');

            item.type = part[0];
            item.required = /^null|optional$/i.test(part[1] || 'required')
                ? false
                : true;

            return item;
        }

        // regexp validator
        if (util.isRegExp(validator)) {
            item.type = 'regexp';
            item.required = !/\^\$/.test(validator.toString());

            return item;
        }

        // lambda validator
        if (util.isFunction(validator)) {
            item.type = 'lambda';
            item.required = true;

            return item;
        }

        throw new Error(util.format('Unsupported validator type: %s', typeof validator));
    }

    /**
     * Identifies whether the specified object holds any value
     * 
     * @param {any} value
     * @returns {Boolean}
     */
    private _isEmpty(value: any): boolean
    {
        if (util.isRegExp(value)) return false;
        if (util.isNullOrUndefined(value)) return true;
        if (util.isString(value) && /^$/.test(value)) return true;
        if (util.isArray(value) && !value.length) return true;
        if (util.isObject(value) && !Object.keys(value).length) return true;
        return false;
    }

    /**
     * Resolves the value of path
     */
    private _resolve(source: any, path: string, need: boolean): any
    {
        // direct resolve
        if (!util.isUndefined(source[path])) return source[path];

        // path resolve
        let value: any = source;
        let parts: string[] = path.split('.');
        let part: string;
        let i: string;

        for (i in parts) {
            part = parts[i];
            if (util.isUndefined(value[part])) {
                if (need) this.error[path] = util.format(`No such property: ${part}`);
                return undefined;
            }
            value = value[part];
        } 

        return value;
    }
}

==> node-validator-tool-1.1.1.tgz: package/src/index.ts <==
'use strict';

// node modules
import * as util from 'util';

interface MetaInfo
{
    required: boolean;
    type: string;
}

export type Dictionary<T> = { [name: string]: T };
export type ValidationFunction = (value: string) => boolean;
export type RuleValidator = RegExp | ValidationFunction | String;

/**
 * Module factory
 */
export function create(rules: Dictionary<RuleValidator>): Validator
{
    return new Validator(rules);
}

export class Validator
{
    public namedRules: Dictionary<RuleValidator> = {
        'uuid': /^([a-z0-9]{8}-)([a-z0-9]{4}-){3}([a-z0-9]{12})$/,
        'email': /\S+@\S+/,
        'datetime': /^\d{4}-\d{2}-\d{2}\s\d{2}:\d{2}:\d{2}(\.\d+)?$/,
        'text': /^[\S\s]+$/,
        'number': /^\d+$/,
        'bool': /^t(rue)?|f(alse)?$/i,
        'abbr': /^[A-Z]+$/,
        'alphanum': /^[a-z0-9]+$/i,
        'gender': /^[mf]$/i,
        'zip': /^[0-9\s]+$/i,
        'phone': /^\+?[0-9\s]+$/,
        'countrycode': /^[A-Z]{2}$/,
    };
    public rules: Dictionary<RuleValidator> = {};
    public error: Dictionary<string> = {};

    /**
     * Module constructor
     */
    constructor(rules?: Dictionary<RuleValidator>)
    {
        // add custom rules
        if (util.isObject(rules)) {
            for (let name in rules) {
                this.addRule(name, rules[name]);
            }
        }
    }

    /**
     * Validation method
     */
    public validate(value: any, validator: string | RuleValidator): boolean
    {
        // no validation needed
        if (this._isEmpty(validator)) {
            return true;
        }

        let item = this._meta(validator);

        // skip empty value
        if (this._isEmpty(value)) {
            return !item.required;
        }

        // validation
        switch (item.type) {
            case 'regexp':
                return (validator as RegExp).test(value);

            case 'lambda':
                return (validator as ValidationFunction)(value);

            default:
                let rule = this.rules[item.type] || this.namedRules[item.type];

                if (rule === undefined) {
                    throw new Error(util.format('Unsupported validation type: %s', item.type));
                }

                if (util.isRegExp(rule)) {
                    return (rule as RegExp).test(value);
                }

                if (util.isFunction(rule)) {
                    return (rule as ValidationFunction)(value);
                }
        }
    }

    /**
     * Run all rules against respective item properties
     */
    public validateAll(item: any): boolean
    {
        this.error = {};

        let name: string,
            validator: RuleValidator,
            required: boolean,
            value: any,
            pass: boolean = true;

        for (name in this.rules) {
            validator = this.rules[name];
            required = this._meta(validator).required;
            value = this._resolve(item, name, required);

            if (this.validate(value, validator) === false) {
                this.error[name] = (value === undefined)
                    ? `Missing property: ${name}`
                    : `Validation error: given "${value}", expected "${(typeof validator).match('function') ? 'something else' : validator}"`;
                pass = false;
            }
        }

        return pass;
    }

    public hasErrors(): boolean
    {
        return Object.keys(this.error).length > 0;
    }

    public getErrors(): Dictionary<string>
    {
        return this.error;
    }

    /**
     * Adds new rule
     */
    public addRule(name: string, validator: RuleValidator): Validator
    {
        if (!util.isString(name)) {
            throw new Error(util.format('Parameter "name" must be string, %s given', typeof name));
        }

        if (util.isNull(validator)) {
            return this;    // skip
        }

        if (!util.isRegExp(validator) && !util.isFunction(validator) && !util.isString(validator)) {
            throw new Error(util.format('Parameter "validator" must be either RegExp, Function or String (named validator), %s given', typeof validator));
        }

        const meta = this._meta(validator);

        if (util.isString(validator) && this.namedRules[meta.type] === undefined) {
            throw new Error(`Parameter "validator": invalid named validator "${validator}"`);
        }

        this.rules[name] = util.isString(validator)
            ? this._getNamedRule(meta)
            : validator;

        return this;
    }

    /**
     * Removes a rule
     */
    public removeRule(name: string): Validator
    {
        delete this.rules[name];
        return this;
    }

    /**
     * Returns whether this validator contains a rule of the specified name
     */
    public hasRule(name: string): boolean
    {
        return !!this.rules[name];
    }


    // Helper functions

    /**
     * Returns named rule
     */
    private _getNamedRule(item: MetaInfo): RuleValidator
    {
        return item.type + (item.required ? '' : ':null');
    }

    /**
     * Parse meta info from supplied type
     */
    private _meta(validator: string | RuleValidator): MetaInfo
    {
        let item: MetaInfo = {
            type: null,
            required: null,
        };

        // validation not needed
        if (this._isEmpty(validator)) {
            return item;
        }

        // named validator
        if (util.isString(validator)) {
            let part = (validator as string).split(':');

            item.type = part[0];
            item.required = /^null|optional$/i.test(part[1] || 'required')
                ? false
                : true;

            return item;
        }

        // regexp validator
        if (util.isRegExp(validator)) {
            item.type = 'regexp';
            item.required = !/\^\$/.test(validator.toString());

            return item;
        }

        // lambda validator
        if (util.isFunction(validator)) {
            item.type = 'lambda';
            item.required = true;

            return item;
        }

        throw new Error(util.format('Unsupported validator type: %s', typeof validator));
    }

    /**
     * Identifies whether the specified object holds any value
     *
     * @param {any} value
     * @returns {Boolean}
     */
    private _isEmpty(value: any): boolean
    {
        if (util.isRegExp(value)) return false;
        if (util.isNullOrUndefined(value)) return true;
        if (util.isString(value) && /^$/.test(value)) return true;
        if (util.isArray(value) && !value.length) return true;
        if (util.isObject(value) && !Object.keys(value).length) return true;
        return false;
    }

    /**
     * Resolves the value of path
     */
    private _resolve(source: any, path: string, need: boolean): any
    {
        // direct resolve
        if (!util.isUndefined(source[path])) return source[path];

        // path resolve
        let value: any = source;
        let parts: string[] = path.split('.');
        let part: string;
        let i: string;

        for (i in parts) {
            part = parts[i];
            if (util.isUndefined(value[part])) {
                if (need) this.error[path] = util.format(`No such property: ${part}`);
                return undefined;
            }
            value = value[part];
        }

        return value;
    }
}
